@@ -1,5 +1,5 @@
 /**
- * Copyright 2005-2013 The Kuali Foundation
+ * Copyright 2005-2014 The Kuali Foundation
  *
  * Licensed under the Educational Community License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,7 +24,10 @@ import org.kuali.rice.krad.document.Document;
  * requiring a {@link DocumentEntry} in the kuali data dictionary
  * 
  * @author Kuali Rice Team (rice.collab@kuali.org)
+ *
+ * @deprecated KNS test class, convert to KRAD equivalent if applicable.
  */
+@Deprecated
 public class PessimisticLockTestDocumentAuthorizer extends DocumentAuthorizerBase {
 
     public static boolean USES_PESSIMISTIC_LOCKING = true;
