@@ -15,10 +15,26 @@
  */
 package org.kuali.rice.krad.document;
 
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+
+import javax.persistence.Column;
+import javax.persistence.Id;
+import javax.persistence.MappedSuperclass;
+import javax.persistence.PostLoad;
+import javax.persistence.PostRemove;
+import javax.persistence.PrePersist;
+import javax.persistence.Transient;
+
 import org.apache.commons.lang.StringUtils;
 import org.apache.log4j.Logger;
+import org.kuali.rice.core.api.mo.common.GloballyUnique;
 import org.kuali.rice.kew.api.KewApiConstants;
 import org.kuali.rice.kew.api.KewApiServiceLocator;
+import org.kuali.rice.kew.api.WorkflowDocument;
+import org.kuali.rice.kew.api.action.ActionRequest;
 import org.kuali.rice.kew.api.action.ActionType;
 import org.kuali.rice.kew.api.exception.WorkflowException;
 import org.kuali.rice.kew.framework.postprocessor.ActionTakenEvent;
@@ -26,19 +42,19 @@ import org.kuali.rice.kew.framework.postprocessor.DocumentRouteLevelChange;
 import org.kuali.rice.kew.framework.postprocessor.DocumentRouteStatusChange;
 import org.kuali.rice.kim.api.identity.Person;
 import org.kuali.rice.kim.api.services.KimApiServiceLocator;
+import org.kuali.rice.krad.UserSessionUtils;
 import org.kuali.rice.krad.bo.AdHocRoutePerson;
 import org.kuali.rice.krad.bo.AdHocRouteWorkgroup;
 import org.kuali.rice.krad.bo.DocumentHeader;
 import org.kuali.rice.krad.bo.Note;
-import org.kuali.rice.krad.bo.PersistableBusinessObject;
-import org.kuali.rice.krad.bo.PersistableBusinessObjectBase;
+import org.kuali.rice.krad.bo.PersistableBusinessObjectBaseAdapter;
 import org.kuali.rice.krad.datadictionary.DocumentEntry;
 import org.kuali.rice.krad.datadictionary.WorkflowAttributes;
 import org.kuali.rice.krad.datadictionary.WorkflowProperties;
 import org.kuali.rice.krad.document.authorization.PessimisticLock;
 import org.kuali.rice.krad.exception.PessimisticLockingException;
 import org.kuali.rice.krad.exception.ValidationException;
-import org.kuali.rice.krad.rules.rule.event.KualiDocumentEvent;
+import org.kuali.rice.krad.rules.rule.event.DocumentEvent;
 import org.kuali.rice.krad.service.AttachmentService;
 import org.kuali.rice.krad.service.DocumentSerializerService;
 import org.kuali.rice.krad.service.KRADServiceLocator;
@@ -49,7 +65,6 @@ import org.kuali.rice.krad.util.GlobalVariables;
 import org.kuali.rice.krad.util.KRADConstants;
 import org.kuali.rice.krad.util.KRADPropertyConstants;
 import org.kuali.rice.krad.util.NoteType;
-import org.kuali.rice.krad.util.ObjectUtils;
 import org.kuali.rice.krad.util.documentserializer.AlwaysFalsePropertySerializabilityEvaluator;
 import org.kuali.rice.krad.util.documentserializer.AlwaysTruePropertySerializibilityEvaluator;
 import org.kuali.rice.krad.util.documentserializer.BusinessObjectPropertySerializibilityEvaluator;
@@ -57,46 +72,34 @@ import org.kuali.rice.krad.util.documentserializer.PropertySerializabilityEvalua
 import org.kuali.rice.krad.workflow.DocumentInitiator;
 import org.kuali.rice.krad.workflow.KualiDocumentXmlMaterializer;
 import org.kuali.rice.krad.workflow.KualiTransactionalDocumentInformation;
-import org.springframework.util.AutoPopulatingList;
 import org.springframework.util.CollectionUtils;
 
-import javax.persistence.CascadeType;
-import javax.persistence.Column;
-import javax.persistence.FetchType;
-import javax.persistence.Id;
-import javax.persistence.JoinColumn;
-import javax.persistence.MappedSuperclass;
-import javax.persistence.OneToMany;
-import javax.persistence.OneToOne;
-import javax.persistence.Transient;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-
 /**
  * @see Document
  */
 @MappedSuperclass
-public abstract class DocumentBase extends PersistableBusinessObjectBase implements Document {
+public abstract class DocumentBase extends PersistableBusinessObjectBaseAdapter implements Document {
+    private static final long serialVersionUID = 8530945307802647664L;
     private static final Logger LOG = Logger.getLogger(DocumentBase.class);
 
-    @Id @Column(name = "DOC_HDR_ID")
+    @Id
+    @Column(name = "DOC_HDR_ID",length=14)
     protected String documentNumber;
-    @OneToOne(fetch = FetchType.LAZY, cascade = {CascadeType.PERSIST, CascadeType.MERGE}) @JoinColumn(
-            name = "DOC_HDR_ID", insertable = false, updatable = false)
+
+    @Transient
     protected DocumentHeader documentHeader;
 
-    @OneToMany(fetch = FetchType.LAZY, cascade = {CascadeType.PERSIST, CascadeType.MERGE}) @JoinColumn(
-            name = "DOC_HDR_ID", insertable = false, updatable = false)
-    private List<PessimisticLock> pessimisticLocks;
+    @Transient
+    protected List<PessimisticLock> pessimisticLocks;
 
     @Transient
-    private List<AdHocRoutePerson> adHocRoutePersons;
+    protected List<AdHocRoutePerson> adHocRoutePersons;
     @Transient
-    private List<AdHocRouteWorkgroup> adHocRouteWorkgroups;
+    protected List<AdHocRouteWorkgroup> adHocRouteWorkgroups;
     @Transient
-    private List<Note> notes;
+    protected List<Note> notes;
+    @Transient
+    private String superUserAnnotation = "";
 
     private transient NoteService noteService;
     private transient AttachmentService attachmentService;
@@ -105,27 +108,18 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      * Constructs a DocumentBase.java.
      */
     public DocumentBase() {
-        try {
-            // create a new document header object
-            Class<? extends DocumentHeader> documentHeaderClass =
-                    KRADServiceLocatorWeb.getDocumentHeaderService().getDocumentHeaderBaseClass();
-            setDocumentHeader(documentHeaderClass.newInstance());
-            pessimisticLocks = new ArrayList<PessimisticLock>();
-            adHocRoutePersons = new ArrayList<AdHocRoutePerson>();
-            adHocRouteWorkgroups = new ArrayList<AdHocRouteWorkgroup>();
-            notes = new ArrayList<Note>();
-        } catch (IllegalAccessException e) {
-            throw new RuntimeException("Error instantiating DocumentHeader", e);
-        } catch (InstantiationException e) {
-            throw new RuntimeException("Error instantiating DocumentHeader", e);
-        }
+        documentHeader = new DocumentHeader();
+        pessimisticLocks = new ArrayList<PessimisticLock>();
+        adHocRoutePersons = new ArrayList<AdHocRoutePerson>();
+        adHocRouteWorkgroups = new ArrayList<AdHocRouteWorkgroup>();
+        notes = new ArrayList<Note>();
     }
 
     /**
      * @see org.kuali.rice.krad.document.Document#getAllowsCopy()
      */
+    @Override
     public boolean getAllowsCopy() {
-        // TODO Auto-generated method stub
         return false;
     }
 
@@ -144,6 +138,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#getDocumentTitle()
      */
+    @Override
     public String getDocumentTitle() {
         String documentTypeLabel = KewApiServiceLocator.getDocumentTypeService().getDocumentTypeByName(
                 this.getDocumentHeader().getWorkflowDocument().getDocumentTypeName()).getLabel();
@@ -159,42 +154,10 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
         return documentTypeLabel + " - " + description;
     }
 
-    /**
-     * Uses the persistence service's implementation of OJB's retrieveNonKey() fields method.
-     *
-     * @see org.kuali.rice.krad.bo.BusinessObject#refresh()
-     */
-    @Override
-    public void refresh() {
-        KRADServiceLocator.getPersistenceService().retrieveNonKeyFields(this);
-    }
-
-    /**
-     * Checks to see if the objectId value is empty. If so, it will try to refresh the object from the DB.
-     *
-     * @see org.kuali.rice.krad.document.Document#refreshIfEmpty()
-     */
-    public void refreshIfEmpty() {
-        if (null == this.getDocumentHeader()) {
-            this.refresh();
-        } else if (StringUtils.isEmpty(this.getDocumentHeader().getObjectId())) {
-            this.refresh();
-        }
-    }
-
-    /**
-     * Uses the persistence service to retrieve a reference object of a parent.
-     *
-     * @see org.kuali.rice.krad.document.Document#refreshReferenceObject(java.lang.String)
-     */
-    @Override
-    public void refreshReferenceObject(String referenceObjectName) {
-        KRADServiceLocator.getPersistenceService().retrieveReferenceObject(this, referenceObjectName);
-    }
-
     /**
      * @see org.kuali.rice.krad.document.Document#prepareForSave()
      */
+    @Override
     public void prepareForSave() {
         // do nothing
     }
@@ -202,6 +165,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
     /**
      * @see org.kuali.rice.krad.document.Document#processAfterRetrieve()
      */
+    @Override
     public void processAfterRetrieve() {
         // do nothing
     }
@@ -213,6 +177,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#doRouteLevelChange(org.kuali.rice.kew.framework.postprocessor.DocumentRouteLevelChange)
      */
+    @Override
     public void doRouteLevelChange(DocumentRouteLevelChange levelChangeEvent) {
         // do nothing
     }
@@ -220,6 +185,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
     /**
      * @see org.kuali.rice.krad.document.Document#doActionTaken(org.kuali.rice.kew.framework.postprocessor.ActionTakenEvent)
      */
+    @Override
     public void doActionTaken(ActionTakenEvent event) {
         if ((KRADServiceLocatorWeb.getDataDictionaryService().getDataDictionary().getDocumentEntry(
                 this.getClass().getName()).getUseWorkflowPessimisticLocking()) && (!getNonLockingActionTakenCodes()
@@ -231,10 +197,17 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
     /**
      * @see org.kuali.rice.krad.document.Document#afterActionTaken(ActionType, org.kuali.rice.kew.framework.postprocessor.ActionTakenEvent)
      */
+    @Override
     public void afterActionTaken(ActionType performed, ActionTakenEvent event) {
         // do nothing
     }
 
+    /**
+     * Return the list of actions a user could take on a document which should not result
+     * in the recalculation of the {@link PessimisticLock}s.
+     * 
+     * @see #doActionTaken(ActionTakenEvent)
+     */
     protected List<String> getNonLockingActionTakenCodes() {
         List<String> actionTakenStatusCodes = new ArrayList<String>();
         actionTakenStatusCodes.add(KewApiConstants.ACTION_TAKEN_SAVED_CD);
@@ -252,6 +225,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#afterWorkflowEngineProcess(boolean)
      */
+    @Override
     public void afterWorkflowEngineProcess(boolean successfullyProcessed) {
         if (KRADServiceLocatorWeb.getDataDictionaryService().getDataDictionary().getDocumentEntry(
                 this.getClass().getName()).getUseWorkflowPessimisticLocking()) {
@@ -267,6 +241,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#beforeWorkflowEngineProcess()
      */
+    @Override
     public void beforeWorkflowEngineProcess() {
         // do nothing
     }
@@ -276,6 +251,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#getWorkflowEngineDocumentIdsToLock()
      */
+    @Override
     public List<String> getWorkflowEngineDocumentIdsToLock() {
         return null;
     }
@@ -290,8 +266,6 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
         String sourceDocumentHeaderId = getDocumentNumber();
         setNewDocumentHeader();
 
-        getDocumentHeader().setDocumentTemplateNumber(sourceDocumentHeaderId);
-
         //clear out notes from previous bo
         this.notes.clear();
         addCopyErrorDocumentNote("copied from document " + sourceDocumentHeaderId);
@@ -303,23 +277,35 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      * @throws WorkflowException
      */
     protected void setNewDocumentHeader() throws WorkflowException {
-        TransactionalDocument newDoc =
-                (TransactionalDocument) KRADServiceLocatorWeb.getDocumentService().getNewDocument(
-                        getDocumentHeader().getWorkflowDocument().getDocumentTypeName());
-        newDoc.getDocumentHeader().setDocumentDescription(getDocumentHeader().getDocumentDescription());
-        newDoc.getDocumentHeader().setOrganizationDocumentNumber(getDocumentHeader().getOrganizationDocumentNumber());
+        // collect the header information from the old document
+        Person user = GlobalVariables.getUserSession().getPerson();
+        WorkflowDocument sourceWorkflowDocument
+                = KRADServiceLocatorWeb.getWorkflowDocumentService().loadWorkflowDocument(getDocumentNumber(), user);
+        String sourceDocumentTypeName = sourceWorkflowDocument.getDocumentTypeName();
 
+        // initiate the new workflow entry, get the workflow doc
+        WorkflowDocument workflowDocument
+                = KRADServiceLocatorWeb.getWorkflowDocumentService().createWorkflowDocument(sourceDocumentTypeName, user);
+        UserSessionUtils.addWorkflowDocument(GlobalVariables.getUserSession(), workflowDocument);
+
+        // set new values on the document header, including the document number from which it was copied
+        Document newDocument = KRADServiceLocatorWeb.getDocumentService().getNewDocument(sourceDocumentTypeName);
+        DocumentHeader newDocumentHeader = newDocument.getDocumentHeader();
+        newDocumentHeader.setDocumentTemplateNumber(getDocumentNumber());
+        newDocumentHeader.setDocumentDescription(getDocumentHeader().getDocumentDescription());
+        newDocumentHeader.setOrganizationDocumentNumber(getDocumentHeader().getOrganizationDocumentNumber());
+
+        // set the new document number on this document
         try {
-            ObjectUtils.setObjectPropertyDeep(this, KRADPropertyConstants.DOCUMENT_NUMBER, documentNumber.getClass(),
-                    newDoc.getDocumentNumber());
+            KRADServiceLocatorWeb.getLegacyDataAdapter().setObjectPropertyDeep(this,
+                    KRADPropertyConstants.DOCUMENT_NUMBER, documentNumber.getClass(), newDocument.getDocumentNumber());
         } catch (Exception e) {
-            LOG.error("Unable to set document number property in copied document " + e.getMessage(), e);
-            throw new RuntimeException("Unable to set document number property in copied document " + e.getMessage(),
-                    e);
+            LOG.error("Unable to set document number property in copied document " + this, e);
+            throw new RuntimeException("Unable to set document number property in copied document " + this, e);
         }
 
-        // replace current documentHeader with new documentHeader
-        setDocumentHeader(newDoc.getDocumentHeader());
+        // replace the current document header with the new document header
+        setDocumentHeader(newDocument.getDocumentHeader());
     }
 
     /**
@@ -341,6 +327,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
     /**
      * @see org.kuali.rice.krad.document.Document#getXmlForRouteReport()
      */
+    @Override
     public String getXmlForRouteReport() {
         prepareForSave();
         populateDocumentForRouting();
@@ -350,6 +337,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
     /**
      * @see org.kuali.rice.krad.document.Document#populateDocumentForRouting()
      */
+    @Override
     public void populateDocumentForRouting() {
         getDocumentHeader().getWorkflowDocument().setApplicationContent(serializeDocumentToXml());
     }
@@ -357,6 +345,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
     /**
      * @see org.kuali.rice.krad.document.Document#serializeDocumentToXml()
      */
+    @Override
     public String serializeDocumentToXml() {
         DocumentSerializerService documentSerializerService = KRADServiceLocatorWeb.getDocumentSerializerService();
         String xml = documentSerializerService.serializeDocumentToXmlForRouting(this);
@@ -369,6 +358,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#wrapDocumentWithMetadataForXmlSerialization()
      */
+    @Override
     public KualiDocumentXmlMaterializer wrapDocumentWithMetadataForXmlSerialization() {
         KualiTransactionalDocumentInformation transInfo = new KualiTransactionalDocumentInformation();
         DocumentInitiator initiator = new DocumentInitiator();
@@ -391,6 +381,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#getDocumentPropertySerizabilityEvaluator()
      */
+    @Override
     public PropertySerializabilityEvaluator getDocumentPropertySerizabilityEvaluator() {
         String docTypeName = getDocumentHeader().getWorkflowDocument().getDocumentTypeName();
         DocumentEntry documentEntry =
@@ -419,67 +410,83 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#getBasePathToDocumentDuringSerialization()
      */
+    @Override
     public String getBasePathToDocumentDuringSerialization() {
         return "document";
     }
 
     /**
-     * @see org.kuali.rice.krad.document.Document#getDocumentHeader()
+     * {@inheritDoc}
      */
+    @Override
     public DocumentHeader getDocumentHeader() {
+        // during the transition time between OJB and JPA - the OJB hooks are not firing
+        // so, we lazy load the document header.
+        if ((documentHeader == null || documentHeader.getDocumentNumber() == null) && StringUtils.isNotBlank(documentNumber)) {
+            documentHeader = KRADServiceLocatorWeb.getDocumentHeaderService().getDocumentHeaderById(documentNumber);
+        }
+
         return this.documentHeader;
     }
 
     /**
-     * @see org.kuali.rice.krad.document.Document#setDocumentHeader(org.kuali.rice.krad.document.DocumentHeader)
+     * {@inheritDoc}
      */
+    @Override
     public void setDocumentHeader(DocumentHeader documentHeader) {
         this.documentHeader = documentHeader;
     }
 
     /**
-     * @see org.kuali.rice.krad.document.Document#getDocumentNumber()
+     * {@inheritDoc}
      */
+    @Override
     public String getDocumentNumber() {
         return documentNumber;
     }
 
     /**
-     * @see org.kuali.rice.krad.document.Document#setDocumentNumber(java.lang.String)
+     * {@inheritDoc}
      */
+    @Override
     public void setDocumentNumber(String documentNumber) {
         this.documentNumber = documentNumber;
     }
 
     /**
-     * @see org.kuali.rice.krad.document.Document#getAdHocRoutePersons()
+     * {@inheritDoc}
      */
+    @Override
     public List<AdHocRoutePerson> getAdHocRoutePersons() {
         return adHocRoutePersons;
     }
 
     /**
-     * @see org.kuali.rice.krad.document.Document#setAdHocRoutePersons(java.util.List)
+     * {@inheritDoc}
      */
+    @Override
     public void setAdHocRoutePersons(List<AdHocRoutePerson> adHocRoutePersons) {
         this.adHocRoutePersons = adHocRoutePersons;
     }
 
     /**
-     * @see org.kuali.rice.krad.document.Document#getAdHocRouteWorkgroups()
+     * {@inheritDoc}
      */
+    @Override
     public List<AdHocRouteWorkgroup> getAdHocRouteWorkgroups() {
         return adHocRouteWorkgroups;
     }
 
     /**
-     * @see org.kuali.rice.krad.document.Document#setAdHocRouteWorkgroups(java.util.List)
+     * {@inheritDoc}
      */
+    @Override
     public void setAdHocRouteWorkgroups(List<AdHocRouteWorkgroup> adHocRouteWorkgroups) {
         this.adHocRouteWorkgroups = adHocRouteWorkgroups;
     }
 
-    public void postProcessSave(KualiDocumentEvent event) {
+    @Override
+    public void postProcessSave(DocumentEvent event) {
         // TODO Auto-generated method stub
 
     }
@@ -487,13 +494,15 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
     /**
      * Override this method with implementation specific prepareForSave logic
      *
-     * @see org.kuali.rice.krad.document.Document#prepareForSave(org.kuali.rice.krad.rules.rule.event.KualiDocumentEvent)
+     * @see org.kuali.rice.krad.document.Document#prepareForSave(org.kuali.rice.krad.rules.rule.event.DocumentEvent)
      */
-    public void prepareForSave(KualiDocumentEvent event) {
+    @Override
+    public void prepareForSave(DocumentEvent event) {
         // do nothing by default
     }
 
-    public void validateBusinessRules(KualiDocumentEvent event) {
+    @Override
+    public void validateBusinessRules(DocumentEvent event) {
         if (GlobalVariables.getMessageMap().hasErrors()) {
             logErrors();
             throw new ValidationException("errors occured before business rule");
@@ -526,15 +535,15 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
         if (LOG.isInfoEnabled()) {
             if (GlobalVariables.getMessageMap().hasErrors()) {
 
-                for (Iterator<Map.Entry<String, AutoPopulatingList<ErrorMessage>>> i =
+                for (Iterator<Map.Entry<String, List<ErrorMessage>>> i =
                              GlobalVariables.getMessageMap().getAllPropertiesAndErrors().iterator(); i.hasNext(); ) {
-                    Map.Entry<String, AutoPopulatingList<ErrorMessage>> e = i.next();
+                    Map.Entry<String, List<ErrorMessage>> e = i.next();
 
                     StringBuffer logMessage = new StringBuffer();
                     logMessage.append("[" + e.getKey() + "] ");
                     boolean first = true;
 
-                    AutoPopulatingList<ErrorMessage> errorList = e.getValue();
+                    List<ErrorMessage> errorList = e.getValue();
                     for (Iterator<ErrorMessage> j = errorList.iterator(); j.hasNext(); ) {
                         ErrorMessage em = j.next();
 
@@ -557,13 +566,15 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#generateSaveEvents()
      */
-    public List<KualiDocumentEvent> generateSaveEvents() {
-        return new ArrayList<KualiDocumentEvent>();
+    @Override
+    public List<DocumentEvent> generateSaveEvents() {
+        return new ArrayList<DocumentEvent>();
     }
 
     /**
      * @see org.kuali.rice.krad.document.Document#doRouteStatusChange(org.kuali.rice.kew.framework.postprocessor.DocumentRouteStatusChange)
      */
+    @Override
     public void doRouteStatusChange(DocumentRouteStatusChange statusChangeEvent) {
         // do nothing
     }
@@ -579,7 +590,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      * @return Returns the documentBusinessObject.
      */
     @Override
-    public PersistableBusinessObject getNoteTarget() {
+    public GloballyUnique getNoteTarget() {
         return getDocumentHeader();
     }
 
@@ -653,28 +664,76 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
         this.notes = notes;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     @Override
+    public List<ActionRequest> getActionRequests() {
+        return KewApiServiceLocator.getWorkflowDocumentService().getPendingActionRequests(getDocumentNumber());
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public String getSuperUserAnnotation() {
+        return superUserAnnotation;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void setSuperUserAnnotation(String superUserAnnotation) {
+        this.superUserAnnotation = superUserAnnotation;
+    }
+
+    /**
+     * Loads the KRAD document header via the document header service.
+     *
+     * @see org.kuali.rice.krad.bo.PersistableBusinessObjectBase#postLoad()
+     */
+    @PostLoad
     protected void postLoad() {
-        super.postLoad();
+        documentHeader = KRADServiceLocatorWeb.getDocumentHeaderService().getDocumentHeaderById(documentNumber);
         refreshPessimisticLocks();
     }
 
+    /**
+     * Save the KRAD document header via the document header service.
+     */
+    @PrePersist
+    protected void prePersist() {
+        super.prePersist();
+        // KRAD/JPA - have to change the handle to object to that just saved
+        documentHeader = KRADServiceLocatorWeb.getDocumentHeaderService().saveDocumentHeader(documentHeader);
+    }
+
+    /**
+     * This overridden method is used to delete the {@link DocumentHeader} object due to the system not being able to
+     * manage the {@link DocumentHeader} object via mapping files
+     *
+     * @see org.kuali.rice.krad.bo.PersistableBusinessObjectBase#postRemove()
+     */
+    @PostRemove
+    protected void postRemove() {
+        KRADServiceLocatorWeb.getDocumentHeaderService().deleteDocumentHeader(getDocumentHeader());
+    }
+
     /**
      * @see org.kuali.rice.krad.document.Document#getPessimisticLocks()
      */
+    @Override
     public List<PessimisticLock> getPessimisticLocks() {
-        return this.pessimisticLocks;
+        return pessimisticLocks;
     }
 
     /**
      * @see org.kuali.rice.krad.document.Document#refreshPessimisticLocks()
-     * @deprecated This is not needed with the relationship set up with JPA annotations
      */
-    @Deprecated
+    @Override
     public void refreshPessimisticLocks() {
-        this.pessimisticLocks.clear();
-        this.pessimisticLocks = KRADServiceLocatorWeb.getPessimisticLockService().getPessimisticLocksForDocument(
-                this.documentNumber);
+        pessimisticLocks = KRADServiceLocatorWeb.getPessimisticLockService().getPessimisticLocksForDocument(documentNumber);
     }
 
     /**
@@ -687,13 +746,25 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
     /**
      * @see org.kuali.rice.krad.document.Document#addPessimisticLock(org.kuali.rice.krad.document.authorization.PessimisticLock)
      */
+    @Override
     public void addPessimisticLock(PessimisticLock lock) {
-        this.pessimisticLocks.add(lock);
+        pessimisticLocks.add(lock);
     }
 
     /**
-     * @see org.kuali.rice.krad.document.Document#getLockClearningMethodNames()
+     * @see org.kuali.rice.krad.document.Document#getLockClearingMethodNames()
      */
+    @Override
+    @Deprecated
+    public List<String> getLockClearingMethodNames() {
+        return getLockClearningMethodNames();
+    }
+
+    /**
+     * @see org.kuali.rice.krad.document.Document#getLockClearingMethodNames()
+     */
+    @Override
+    @Deprecated
     public List<String> getLockClearningMethodNames() {
         List<String> methodToCalls = new ArrayList<String>();
         methodToCalls.add(KRADConstants.CLOSE_METHOD);
@@ -713,6 +784,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#useCustomLockDescriptors()
      */
+    @Override
     public boolean useCustomLockDescriptors() {
         return false;
     }
@@ -724,6 +796,7 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
      *
      * @see org.kuali.rice.krad.document.Document#getCustomLockDescriptor(org.kuali.rice.kim.api.identity.Person)
      */
+    @Override
     public String getCustomLockDescriptor(Person user) {
         throw new PessimisticLockingException("Document " + getDocumentNumber() +
                 " is using pessimistic locking with custom lock descriptors, but the document class has not overriden the getCustomLockDescriptor method");
@@ -742,4 +815,22 @@ public abstract class DocumentBase extends PersistableBusinessObjectBase impleme
         }
         return noteService;
     }
+
+    /**
+     * Overrides this OJB method to accept the no-longer-bound documentHeader reference
+     * and perform the refresh via services instead of via OJB.
+     *
+     * For any other property, it works as before.
+     *
+     * @deprecated This is a KNS/OJB-related method.  It should not be used on KRAD/JPA-based documents.
+     * @see org.kuali.rice.krad.bo.PersistableBusinessObjectBase#refreshReferenceObject(java.lang.String)
+     */
+    @Deprecated
+    public void refreshReferenceObject(String referenceObjectName) {
+        if ( StringUtils.equals( referenceObjectName, "documentHeader" ) ) {
+            documentHeader = KRADServiceLocatorWeb.getDocumentHeaderService().getDocumentHeaderById(documentNumber);
+        } else {
+            super.refreshReferenceObject(referenceObjectName);
+        }
+    }
 }
