@@ -14,10 +14,6 @@
 -- limitations under the License.
 --
 
---
--- KULRICE-6916 KRIM_ENTITY_CACHE_T.PRSN_NM is too small
---
-
-ALTER TABLE `krim_entity_cache_t`
-CHANGE COLUMN `PRSN_NM` `PRSN_NM` VARCHAR(255)
-CHARACTER SET 'utf8' COLLATE 'utf8_bin' NULL DEFAULT NULL;
+insert into KRCR_CMPNT_T (NMSPC_CD, CMPNT_CD, NM, ACTV_IND, OBJ_ID, VER_NBR)
+VALUES ('KR-WKFLW', 'Rule', 'Rule', 'Y', uuid(), 1);
+update KRCR_CMPNT_T set cmpnt_cd='EDocLite' where cmpnt_cd like 'EDocLite%';
\ No newline at end of file
