@@ -14,35 +14,17 @@
 -- limitations under the License.
 --
 
--- 
--- mysql-2010-04-15.sql
--- 
+-------------------------------------------------------------------------------------------
+-------------------------------------------------------------------------------------------
+--
+--  Please see README.TXT for special instructions about the 2.2.0 to 2.2.1 upgrade
+--
+-------------------------------------------------------------------------------------------
+-------------------------------------------------------------------------------------------
 
 
-ALTER TABLE krns_sesn_doc_t ADD (OBJ_ID VARCHAR(36))
-/
-ALTER TABLE krns_sesn_doc_t ADD (VER_NBR DECIMAL(8) DEFAULT 0)
-/
-
-
-
-
--- 
--- mysql-2011-06-06.sql
--- 
-
-
-ALTER TABLE KRSB_MSG_QUE_T CHANGE SVC_NMSPC APPL_ID VARCHAR(255)
-/
-
-
-
--- 
--- mysql-2012-01-19c.sql
--- 
-
 --
--- KULRICE-5856: KRNS_DOC_HDR_T.FDOC_DESC column is only 40 char
+-- KULRICE-8573: Add session id to locks and delete these locks when session is destroyed.
 --
 
-ALTER TABLE KRNS_DOC_HDR_T MODIFY FDOC_DESC varchar(255);
+ALTER TABLE KRNS_PESSIMISTIC_LOCK_T ADD COLUMN SESN_ID VARCHAR(40) NOT NULL DEFAULT '';
